import re


class N_INSEE:
    # Similar to 'static initializer' in Java:
    _Expression_reguliere_N_INSEE = re.compile("^\d{13}$",
                                               re.ASCII)  # 're.ASCII' only allows [0-9] in excluding UNICODE digits

    # Improve regular expression as follows (https://fr.wikipedia.org/wiki/Num%C3%A9ro_de_s%C3%A9curit%C3%A9_sociale_en_France):
    # One digit sex between '1' and '2'
    # 2 digits birth year between '01' and '99'
    # 2 digits birth month between '01' and '12'
    # Etc

    @staticmethod
    def Format_N_INSEE():
        return N_INSEE._Expression_reguliere_N_INSEE

    def __init__(self, valeur, clef):
        assert clef >= 1 and clef <= 97  # Pre-condition
        # print(re.findall('^\d{13}$', valeur))
        self._valeur = None
        if N_INSEE.Format_N_INSEE().match(valeur) is not None:
            self._valeur = int(valeur)

    def __eq__(self, n_insee): # Python 3 *IMPLICITELY* implements '__ne__(self, n_insee)' while Python 2 *DOESN'T*!
        return self.__class__ == n_insee.__class__ and self._valeur == n_insee._valeur

    def __hash__(self):
        return hash(self._valeur)

    def clef(self):
        if self._valeur is None:
            raise Exception("Le N°INSEE n'est pas conforme au format " + '"^\d{13}$"')
        return 97 - self._valeur % 97

# Test program, which is incompatible with the reuse of the 'N_INSEE' class in another program;
# try:
#     n = N_INSEE("1630125388055", 29)
#     print('Clef calculée : ', n.clef())
# except AssertionError as ae:
#     print('clef < 1 or clef > 97')
# except Exception as e:
#     print(e)
